/**
 * Testingey
 * Copyright(c) 2011 Jesus A. Domingo <jesus.domingo@gmail.com>
 * MIT Licensed
 */

const sys = require('sys'),
      EventEmitter = require('events').EventEmitter;

function _e(msg) {
  console.log('\033[31m' + msg + '\033[0m');
}

function _s(msg) {
  console.log('\033[32m' + msg + '\033[0m');
}

function _i(msg) {
  console.log('INFO: ' + msg);
}

/**
 * Represents an individual test case
 */
function Runner(task) {
  
  if (typeof task.cb != 'function') {
    throw new Error('Tests need to be enclosed in functions.');
  }
  
  this.task = task;

  this.error = null;

  this.startTime = 0;
  this.endTime = 0;
}

/**
 * We'll be sending done events to the suite
 */
sys.inherits(Runner, EventEmitter);

/**
 * Used to wrap test points in asynchronous test
 * cases
 * @param {Function} function-enclosed test points
 * @api public
 */
Runner.prototype.capture = function (cb) {
  try {
    cb.call(undefined);
  } catch (error) {
    this.error = error;
  }

  this._done();
};

/**
 * Run the test case associated with this runner
 * @api public
 */
Runner.prototype.run = function () {
  
  this.startTime = (new Date()).getTime();

  if (this.task.async) {
    this.task.cb.call(undefined, this);
  } else {
    try {
      this.task.cb.call(undefined, this);
    } catch (error) {
      this.error = error;
    }
    this._done();
  }
};

/**
 * Tell the suite that this case is done
 * @api private
 */
Runner.prototype._done = function () {
  this.endTime = (new Date()).getTime();
  this.emit('done');
};

/**
 * Test suite object
 * @param {String} name for this test suite
 * @api public
 */
function Testingey(name) {
  this._name = name;
  this._tests = [];
  this._hooks = {};
  this._results = [];
  this._pending = 0;
  this._failed = 0;
  this._duration = 0;
}

/**
 * Set a setup function that will be invoked before the
 * tests run.
 * @param {Function} function to invoke during setup
 * @api public
 */
Testingey.prototype.setUp = function (cb) {
  
  if (typeof cb != 'function') {
    throw new Error('A function is expected by Testingey#setUp.');
  }

  this._hooks.setUp = cb;
};

/**
 * Set a cleanup function that will be invoked after the suite is done.
 * @param {Function} function to invoke during setup
 * @api public
 */
Testingey.prototype.tearDown = function (cb) {
  
  if (typeof cb != 'function') {
    throw new Error('A function is expected by Testingey#tearDown.');
  }

  this._hooks.tearDown = cb;
};

/**
 * Add an asynchronous test case to the suite
 * @param {String} description for the test case
 * @param {Function} function containing test code
 * @api public
 */
Testingey.prototype.asyncSpec = function (desc, cb) {
  this._tests.push({ desc: desc, cb: cb, async: true });
};

/**
 * Add a synchronous test case to the suite
 * @param {String} description for the test case
 * @param {Function} function containing test code
 * @api public
 */
Testingey.prototype.spec = function (desc, cb) {
  this._tests.push({ desc: desc, cb: cb, async: false });
};

/**
 * Start running all tests in this suite. Note that
 * all tests are fired in parallel.
 * @api public
 */
Testingey.prototype.run = function () {
  
  var self = this;

  if (this._hooks.setUp) {
    _i('Setting up.');
    this._hooks.setUp.call(undefined);
  }

  _i('Firing all tests, please wait.');
  
  self._duration = (new Date()).getTime(); 
  self._pending = self._tests.length;
  
  self._tests.forEach(function (test) {
    var runner = new Runner(test);
    runner.on('done', function () { self._captureResult(runner); });
    runner.run();
  });
};

/**
 * Record the results of a Runner
 * @param {Object} runner instance to get data from
 * @api private
 */
Testingey.prototype._captureResult = function (runner) {
  
  var result = {
    desc: runner.task.desc,
    duration: runner.endTime - runner.startTime,
    error: runner.error
  };

  this._results.push(result);

  if (result.error != null) {
    _e('FAILED: ' + result.desc + ' (' + result.duration/1000 + 's)\n--' + result.error);
    ++this._failed;
  } else {
    _s('PASSED: ' + result.desc + ' (' + result.duration/1000 + 's)');
  }

  --this._pending;
  
  if (!this._pending) {
    this._duration = ((new Date()).getTime() - this._duration)/1000;
    if (this._hooks.tearDown) {
      _i('Tearing down.');
      this._hooks.tearDown.call(undefined);
    }
    _i('Test suite \"' + this._name + '\" finished in ' + this._duration + 's');
    _i(this._tests.length + ' test(s), ' + this._failed + ' failed');
  }

};

/**
 * Factory method
 */
exports.createSuite = function (name) {
  return new Testingey(name);
};
